@@ -2,13 +2,11 @@ import inspect
 from typing import Dict, List, Optional, Tuple, Union
 
 import torch
-from compressed_tensors.quantization import (
-    disable_quantization,
-    find_name_or_class_matches,
-)
+from compressed_tensors.quantization import disable_quantization
 from compressed_tensors.utils import (
-    align_module_device,
+    align_modules,
     get_execution_device,
+    match_named_modules,
     update_offload_parameter,
 )
 from loguru import logger
@@ -29,7 +27,7 @@ from llmcompressor.modifiers.utils.hooks import HooksMixin
 from llmcompressor.pipelines.cache import IntermediatesCache
 from llmcompressor.utils.fsdp.helpers import get_fsdp_parent
 from llmcompressor.utils.helpers import calibration_forward_context
-from llmcompressor.utils.pytorch.module import get_layer_by_name, get_layers
+from llmcompressor.utils.pytorch.module import get_layer_by_name
 
 __all__ = ["AWQModifier"]
 
@@ -234,8 +232,10 @@ class AWQModifier(Modifier, QuantizationMixin):
         # register quantization calibration hooks
         # assume quantization has been initialized by this modifier or one before it
         QuantizationMixin.start_calibration(self, state.model)
-        # Unlike qmod, do not quantize as we calibrate
-        # This choice does not seem to have a meaningful impact on accuracy
+        # AWQ performs forward passes during _apply_smoothing
+        # before any scales or zero points are updated
+        # Quantization must be disabled, otherwise NaNs will
+        # appear in quantized forward method
         state.model.apply(disable_quantization)
 
         self._setup_activation_cache_hooks()
@@ -304,23 +304,17 @@ class AWQModifier(Modifier, QuantizationMixin):
         """
         resolved_mappings: list[ResolvedMapping] = []
         for mapping_idx, mapping in enumerate(self.mappings):
-            smooth_layers = get_layers(mapping.smooth_layer, model)
-            smooth_names = [
-                smooth_name
-                for smooth_name in smooth_layers
-                if not find_name_or_class_matches(
-                    smooth_name, model, self.ignore + ["re:.*_observer$"]
-                )
-            ]
-
             num_skipped_mappings = 0
-            pbar = tqdm(smooth_names)
-            for smooth_name in pbar:
+
+            for smooth_name, smooth_layer in (
+                pbar := tqdm(
+                    match_named_modules(model, [mapping.smooth_layer], self.ignore)
+                )
+            ):
                 pbar.set_description(
                     f"Resolving mapping {mapping_idx+1}/{len(self.mappings)}"
                     f" ({num_skipped_mappings} skipped)"
                 )
-                smooth_layer = smooth_layers[smooth_name]
 
                 smooth_parent_name = ".".join(smooth_name.split(".")[:-1])
                 smooth_parent = get_layer_by_name(smooth_parent_name, model)
@@ -328,10 +322,9 @@ class AWQModifier(Modifier, QuantizationMixin):
                 balance_layers, balance_names = [], []
                 for balance_regex in mapping.balance_layers:
                     # find the submodules that match the activation layer
-                    for balance_suffix, balance_layer in get_layers(
-                        balance_regex,
-                        smooth_parent,
-                    ).items():
+                    for balance_suffix, balance_layer in match_named_modules(
+                        smooth_parent, [balance_regex], self.ignore
+                    ):
                         balance_name = f"{smooth_parent_name}.{balance_suffix}"
 
                         # exclude v_proj->o_proj mappings whose shapes are incompatible
@@ -455,22 +448,26 @@ class AWQModifier(Modifier, QuantizationMixin):
             balance_layers = mapping.balance_layers
             parent_module = mapping.parent
 
-            # [STEP 1]: Compute per-channel mean of normalised weights
-            # All layer weights are concatted together
-            weight = torch.cat([bl.weight for bl in balance_layers], dim=0)
-            org_shape = weight.shape
-            # The weights are reshaped to be organised by quantization group
-            weight = weight.view(-1, self._group_size)
-            # Calculates the relative magnitude of the weights within
-            # each of the quantization groups, and rescales each group
-            # individually so that each group has weights on a 0-1 scale.
-            w_scale = weight.abs() / (weight.abs().amax(dim=1, keepdim=True) + 1e-6)
-            # Resizes the rescaled weight matrix back up to its original dimensions
-            w_scale = w_scale.view(org_shape)
-            # Gets the average rescaled magnitude for each output channel
-            w_mean = w_scale.mean(0)
+            with align_modules(
+                [parent_module, smooth_layer, *balance_layers]
+            ), calibration_forward_context(model), HooksMixin.disable_hooks():
+                # [STEP 1]: Compute per-channel mean of normalised weights
+                # All layer weights are concatted together
+                weight = torch.cat([bl.weight for bl in balance_layers], dim=0)
+                org_shape = weight.shape
+                # The weights are reshaped to be organised by quantization group
+                weight = weight.view(-1, self._group_size)
+                # Calculates the relative magnitude of the weights within
+                # each of the quantization groups, and rescales each group
+                # individually so that each group has weights on a 0-1 scale.
+                weight.abs_()
+                weight.div_(weight.amax(dim=1, keepdim=True) + 1e-6)
+                # Resizes the rescaled weight matrix back up to its original dimensions
+                weight = weight.view(org_shape)
+                # Gets the average rescaled magnitude for each output channel
+                w_mean = weight.mean(0)
+                del weight
 
-            with calibration_forward_context(model), HooksMixin.disable_hooks():
                 # [STEP 3]: Compute output of module
                 # could cache from hook, rather than recomputing here
                 fp16_outputs = self._run_samples(parent_module)
@@ -490,9 +487,8 @@ class AWQModifier(Modifier, QuantizationMixin):
                     x_mean, w_mean, parent_module, balance_layers, fp16_outputs
                 )
 
-            @torch.no_grad()
-            def smooth(module):
-                with align_module_device(module):
+                @torch.no_grad()
+                def _smooth(module):
                     scales = best_scales.to(module.weight.device)
                     if module in balance_layers:
                         update_offload_parameter(
@@ -524,33 +520,31 @@ class AWQModifier(Modifier, QuantizationMixin):
                                 module.bias.div_(scales),
                             )
 
-            parent = get_fsdp_parent(mapping.smooth_name, model)
-            if parent is not None:
-                parent.apply(smooth)
-            else:
-                # if we're not running with FSDP we can apply smoothing directly
-                for layer in balance_layers:
-                    smooth(layer)
-                smooth(smooth_layer)
+                parent = get_fsdp_parent(mapping.smooth_name, model)
+                if parent is not None:
+                    parent.apply(_smooth)
+                else:
+                    # if we're not running with FSDP we can apply smoothing directly
+                    for layer in balance_layers:
+                        _smooth(layer)
+                    _smooth(smooth_layer)
 
-            # remove caches needed to smooth this mapping
-            del self._smooth_activation_means[mapping.smooth_name]
+                # remove caches needed to smooth this mapping
+                del self._smooth_activation_means[mapping.smooth_name]
 
         for v in self._parent_args_cache.values():
             v.batch_intermediates.clear()
         self._assert_all_activations_consumed()
 
     def _run_samples(self, module: Module) -> List[torch.Tensor]:
-        with align_module_device(module):
-            outputs = [
-                module(**batch_kwargs)
-                for batch_kwargs in self._parent_args_cache[module]
-            ]
-            return [
-                # If Tuple, assume that first argument is the input
-                output[0] if isinstance(output, Tuple) else output
-                for output in outputs
-            ]
+        outputs = [
+            module(**batch_kwargs) for batch_kwargs in self._parent_args_cache[module]
+        ]
+        return [
+            # If Tuple, assume that first argument is the input
+            output[0] if isinstance(output, Tuple) else output
+            for output in outputs
+        ]
 
     def _compute_best_scale(
         self,
@@ -575,7 +569,11 @@ class AWQModifier(Modifier, QuantizationMixin):
         best_scales = None
         best_error = float("inf")
 
-        org_sd = {k: v.cpu() for k, v in parent_module.state_dict().items()}
+        org_sd = {
+            k: v.cpu()
+            for k, v in parent_module.state_dict().items()
+            if v.device != torch.device("meta")
+        }
 
         device = get_execution_device(parent_module)
         x_mean = x_mean.view(-1).to(device)
@@ -601,23 +599,21 @@ class AWQModifier(Modifier, QuantizationMixin):
 
             # Q(W * s)
             for linear in linears2scale:
-                with align_module_device(linear):
-                    linear.weight.mul_(_scalesview)
-                    update_offload_parameter(
-                        linear,
-                        "weight",
-                        _pseudo_quantize_tensor(
-                            w=linear.weight.data,
-                            symmetric=self._symmetric,
-                            bit_width=self._num_bits,
-                            group_size=self._group_size,
-                        )[0]
-                        / _scalesview,
-                    )
+                linear.weight.mul_(_scalesview)
+                update_offload_parameter(
+                    linear,
+                    "weight",
+                    _pseudo_quantize_tensor(
+                        w=linear.weight.data,
+                        symmetric=self._symmetric,
+                        bit_width=self._num_bits,
+                        group_size=self._group_size,
+                    )[0]
+                    / _scalesview,
+                )
 
             # W * X
-            with HooksMixin.disable_hooks():
-                int_w_outputs = self._run_samples(parent_module)
+            int_w_outputs = self._run_samples(parent_module)
 
             # compute mean squared error (L2 norm)
             loss = self._compute_loss(fp16_outputs, int_w_outputs, device)
@@ -627,11 +623,17 @@ class AWQModifier(Modifier, QuantizationMixin):
                 best_error = loss
                 best_ratio = ratio
                 best_scales = scales.clone()
-            parent_module.load_state_dict(org_sd)
+
+            parent_module.load_state_dict(org_sd, strict=False)
 
         if best_ratio == -1:
             logger.debug(history)
-            raise Exception
+            raise Exception(
+                "No finite loss was found in best scalesgrid search. This typically "
+                "means NaN values are appearing in the forward pass of the parent "
+                "module. If you encounter this error, raise an issue at "
+                "https://github.com/vllm-project/llm-compressor/issues"
+            )
 
         assert (
             torch.isnan(best_scales).sum() == 0
